"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const deepmerge_1 = tslib_1.__importDefault(require("deepmerge"));
function mergeResolvers(resolversDefinitions) {
    if (!resolversDefinitions || resolversDefinitions.length === 0) {
        return {};
    }
    if (resolversDefinitions.length === 1) {
        return resolversDefinitions[0];
    }
    const resolversFactories = new Array();
    const resolvers = new Array();
    for (const resolversDefinition of resolversDefinitions) {
        if (typeof resolversDefinition === 'function') {
            resolversFactories.push(resolversDefinition);
        }
        else if (typeof resolversDefinition === 'object') {
            resolvers.push(resolversDefinition);
        }
    }
    if (resolversFactories.length) {
        return ((...args) => {
            const resultsOfFactories = resolversFactories.map(factory => factory(...args));
            return deepmerge_1.default.all([...resolvers, ...resultsOfFactories]);
        });
    }
    else {
        return deepmerge_1.default.all(resolvers);
    }
}
exports.mergeResolvers = mergeResolvers;
//# sourceMappingURL=merge-resolvers.js.map