// tslint:disable-next-line:no-reference
/// <reference path="../../node_modules/reflect-metadata/index.d.ts" />
import { GraphQLObjectType, GraphQLList } from 'graphql';
import { DESIGN_PARAMTYPES, DESIGN_TYPE, DESIGN_RETURNTYPE } from './common';
import { getScalarTypeFromClass } from './scalar-type';
import { getInputTypeFromClass } from './input-object-type';
import { asArray } from '../utils/helpers';
export const GRAPHQL_OBJECT_TYPE_CONFIG = Symbol('graphql:object-type-config');
export const GRAPHQL_OBJECT_TYPE = Symbol('graphql:object-type');
export const GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE = Symbol('graphql-object-type-build-queue');
export function Arg(argumentName) {
    return (target, propertyKey, parameterIndex) => {
        const existingGraphQLObjectTypeQueue = Reflect.getMetadata(GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, target.constructor) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const existingConfig = Reflect.getMetadata(GRAPHQL_OBJECT_TYPE_CONFIG, target.constructor) || {};
            const fieldName = propertyKey;
            existingConfig.fields = existingConfig.fields || {};
            existingConfig.fields[fieldName] = existingConfig.fields[fieldName] || {};
            existingConfig.fields[fieldName].args = existingConfig.fields[fieldName].args || {};
            existingConfig.fields[fieldName].args.__defineGetter__(argumentName, () => {
                const argumentType = Reflect.getMetadata(DESIGN_PARAMTYPES, target, propertyKey)[parameterIndex];
                const argumentGraphQLInputType = getInputTypeFromClass(argumentType) || getScalarTypeFromClass(argumentType) || argumentType;
                return {
                    type: argumentGraphQLInputType
                };
            });
            Reflect.defineMetadata(GRAPHQL_OBJECT_TYPE_CONFIG, existingConfig, target.constructor);
        });
        Reflect.defineMetadata(GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, existingGraphQLObjectTypeQueue, target.constructor);
    };
}
export function Field(typeFactory) {
    return (target, propertyKey) => {
        const existingGraphQLObjectTypeQueue = Reflect.getMetadata(GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, target.constructor) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const existingConfig = Reflect.getMetadata(GRAPHQL_OBJECT_TYPE_CONFIG, target.constructor) || {};
            const fieldName = propertyKey;
            existingConfig.fields = existingConfig.fields || {};
            existingConfig.fields[fieldName] = existingConfig.fields[fieldName] || {};
            existingConfig.fields[fieldName].__defineGetter__('type', () => {
                const fieldType = typeFactory ? typeFactory() : (typeof target[propertyKey] === 'function' ? Reflect.getMetadata(DESIGN_RETURNTYPE, target, propertyKey) : Reflect.getMetadata(DESIGN_TYPE, target, propertyKey));
                return getObjectTypeFromClass(fieldType) || getScalarTypeFromClass(fieldType) || fieldType;
            });
            if (typeof target[propertyKey] === 'function') {
                existingConfig.fields[fieldName].resolve = ((root, args) => target[propertyKey].call(root, ...Object['values'](args))); // TODO: NOT SAFE
            }
            Reflect.defineMetadata(GRAPHQL_OBJECT_TYPE_CONFIG, existingConfig, target.constructor);
        });
        Reflect.defineMetadata(GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, existingGraphQLObjectTypeQueue, target.constructor);
    };
}
export function ObjectType(config = {}) {
    return target => {
        const existingGraphQLObjectTypeQueue = Reflect.getMetadata(GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, target) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const existingConfig = Reflect.getMetadata(GRAPHQL_OBJECT_TYPE_CONFIG, target) || {};
            Reflect.defineMetadata(GRAPHQL_OBJECT_TYPE, new GraphQLObjectType(Object.assign({ name: config.name || target.name, interfaces: asArray(config.implements).map(interfaceType => getObjectTypeFromClass(interfaceType) || interfaceType) }, existingConfig)), target);
        });
        Reflect.defineMetadata(GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, existingGraphQLObjectTypeQueue, target);
        return target;
    };
}
export function getObjectTypeFromClass(target) {
    if (target instanceof Array) {
        const elementType = getObjectTypeFromClass(target[0]);
        return elementType && new GraphQLList(elementType);
    }
    if (!Reflect.hasMetadata(GRAPHQL_OBJECT_TYPE, target) &&
        Reflect.hasMetadata(GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, target)) {
        const existingGraphQLObjectTypeQueue = Reflect.getMetadata(GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, target);
        existingGraphQLObjectTypeQueue.forEach(fn => fn());
    }
    return Reflect.getMetadata(GRAPHQL_OBJECT_TYPE, target);
}
//# sourceMappingURL=object-type.js.map