import { validate, specifiedRules } from 'graphql';
const rulesToIgnore = ['KnownFragmentNames', 'NoUnusedFragments', 'NoUnusedVariables', 'KnownDirectives'];
const effectiveRules = specifiedRules.filter((f) => !rulesToIgnore.includes(f.name));
export const validateGraphQlDocuments = (schema, documentFiles) => documentFiles
    .map(result => ({
    filePath: result.filePath,
    errors: validate(schema, result.content, effectiveRules),
}))
    .filter(r => r.errors.length > 0);
export function checkValidationErrors(loadDocumentErrors) {
    if (loadDocumentErrors.length > 0) {
        const errors = [];
        let errorCount = 0;
        for (const loadDocumentError of loadDocumentErrors) {
            for (const graphQLError of loadDocumentError.errors) {
                errors.push(`
          ${loadDocumentError.filePath}: 
            ${graphQLError.message}
        `);
                errorCount++;
            }
        }
        throw new Error(`Found ${errorCount} error${errorCount > 1 ? 's' : ''} in your documents!`);
    }
}
//# sourceMappingURL=validate-documents.js.map