"use strict";
// tslint:disable-next-line:no-reference
/// <reference path="../../node_modules/reflect-metadata/index.d.ts" />
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const common_1 = require("./common");
const scalar_type_1 = require("./scalar-type");
const input_object_type_1 = require("./input-object-type");
const helpers_1 = require("../utils/helpers");
exports.GRAPHQL_OBJECT_TYPE_CONFIG = Symbol('graphql:object-type-config');
exports.GRAPHQL_OBJECT_TYPE = Symbol('graphql:object-type');
exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE = Symbol('graphql-object-type-build-queue');
function Arg(argumentName) {
    return (target, propertyKey, parameterIndex) => {
        const existingGraphQLObjectTypeQueue = Reflect.getMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, target.constructor) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const existingConfig = Reflect.getMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG, target.constructor) || {};
            const fieldName = propertyKey;
            existingConfig.fields = existingConfig.fields || {};
            existingConfig.fields[fieldName] = existingConfig.fields[fieldName] || {};
            existingConfig.fields[fieldName].args = existingConfig.fields[fieldName].args || {};
            existingConfig.fields[fieldName].args.__defineGetter__(argumentName, () => {
                const argumentType = Reflect.getMetadata(common_1.DESIGN_PARAMTYPES, target, propertyKey)[parameterIndex];
                const argumentGraphQLInputType = input_object_type_1.getInputTypeFromClass(argumentType) || scalar_type_1.getScalarTypeFromClass(argumentType) || argumentType;
                return {
                    type: argumentGraphQLInputType
                };
            });
            Reflect.defineMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG, existingConfig, target.constructor);
        });
        Reflect.defineMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, existingGraphQLObjectTypeQueue, target.constructor);
    };
}
exports.Arg = Arg;
function Field(typeFactory) {
    return (target, propertyKey) => {
        const existingGraphQLObjectTypeQueue = Reflect.getMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, target.constructor) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const existingConfig = Reflect.getMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG, target.constructor) || {};
            const fieldName = propertyKey;
            existingConfig.fields = existingConfig.fields || {};
            existingConfig.fields[fieldName] = existingConfig.fields[fieldName] || {};
            existingConfig.fields[fieldName].__defineGetter__('type', () => {
                const fieldType = typeFactory ? typeFactory() : (typeof target[propertyKey] === 'function' ? Reflect.getMetadata(common_1.DESIGN_RETURNTYPE, target, propertyKey) : Reflect.getMetadata(common_1.DESIGN_TYPE, target, propertyKey));
                return getObjectTypeFromClass(fieldType) || scalar_type_1.getScalarTypeFromClass(fieldType) || fieldType;
            });
            if (typeof target[propertyKey] === 'function') {
                existingConfig.fields[fieldName].resolve = ((root, args) => target[propertyKey].call(root, ...Object['values'](args))); // TODO: NOT SAFE
            }
            Reflect.defineMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG, existingConfig, target.constructor);
        });
        Reflect.defineMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, existingGraphQLObjectTypeQueue, target.constructor);
    };
}
exports.Field = Field;
function ObjectType(config = {}) {
    return target => {
        const existingGraphQLObjectTypeQueue = Reflect.getMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, target) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const existingConfig = Reflect.getMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG, target) || {};
            Reflect.defineMetadata(exports.GRAPHQL_OBJECT_TYPE, new graphql_1.GraphQLObjectType(Object.assign({ name: config.name || target.name, interfaces: helpers_1.asArray(config.implements).map(interfaceType => getObjectTypeFromClass(interfaceType) || interfaceType) }, existingConfig)), target);
        });
        Reflect.defineMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, existingGraphQLObjectTypeQueue, target);
        return target;
    };
}
exports.ObjectType = ObjectType;
function getObjectTypeFromClass(target) {
    if (target instanceof Array) {
        const elementType = getObjectTypeFromClass(target[0]);
        return elementType && new graphql_1.GraphQLList(elementType);
    }
    if (!Reflect.hasMetadata(exports.GRAPHQL_OBJECT_TYPE, target) &&
        Reflect.hasMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, target)) {
        const existingGraphQLObjectTypeQueue = Reflect.getMetadata(exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE, target);
        existingGraphQLObjectTypeQueue.forEach(fn => fn());
    }
    return Reflect.getMetadata(exports.GRAPHQL_OBJECT_TYPE, target);
}
exports.getObjectTypeFromClass = getObjectTypeFromClass;
//# sourceMappingURL=object-type.js.map